# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"get_sensor_data": "01_json_to_df.ipynb",
         "addAATData": "01_json_to_df.ipynb",
         "addSessionData": "01_json_to_df.ipynb",
         "makeParticipantTable": "01_json_to_df.ipynb",
         "relabel_variables": "01_json_to_df.ipynb",
         "json_to_df": "01_json_to_df.ipynb",
         "Constants": "02_constants.ipynb",
         "interpolate": "03_add_responses.ipynb",
         "butterFilter": "03_add_responses.ipynb",
         "find_first_peak": "03_add_responses.ipynb",
         "find_response": "03_add_responses.ipynb",
         "getResponses": "03_add_responses.ipynb",
         "add_responses": "03_add_responses.ipynb",
         "merge_data": "04_utils.ipynb",
         "loadJson": "04_utils.ipynb",
         "sorted_nicely": "05_make_condition_templates.ipynb",
         "getProperty": "05_make_condition_templates.ipynb",
         "makeConditionTable": "05_make_condition_templates.ipynb",
         "make_condition_templates_from_files": "05_make_condition_templates.ipynb",
         "make_condition_templates": "05_make_condition_templates.ipynb",
         "BASE_COLUMNS": "05_make_condition_templates.ipynb",
         "AAT_COLUMNS": "05_make_condition_templates.ipynb"}

modules = ["json_to_df.py",
           "constants.py",
           "add_responses.py",
           "utils.py",
           "make_condition_templates.py"]

doc_url = "https://hgzech.github.io/aat_analysis/"

git_url = "https://github.com/hgzech/aat_analysis/tree/master/"

def custom_doc_links(name): return None



# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/02_constants.ipynb (unless otherwise specified).

__all__ = ['Constants']

# Cell
class Constants():
        ACCELERATION_COLUMN = 'accelerations'
        ACCELERATION_X_COLUMN = 'accelerations_x'
        ACCELERATION_Y_COLUMN = 'accelerations_y'
        ACCURACY_COLUMN = 'accuracy'
        BLOCK_COLUMN = "block"
        BUTTERED_COLUMN = 'buttered'
        CONDITION_COLUMN = "condition"
        CORRECT_RESPONSE_COLUMN = 'correct_response'
        DEVICE_COLUMN = 'device'
        DRAWN_AT_COLUMN = 'drawn_at'
        DRAWN_AT_UNIX_COLUMN = 'drawn_at_unix'
        EXPERIMENT_COLUMN = 'experiment'
        FORCE_COLUMN = 'force'
        GYRO_TIME_COLUMN = 'gyro_times'
        GYRO_X_COLUMN = 'gyro_x'
        GYRO_Y_COLUMN = 'gyro_y'
        GYRO_Z_COLUMN = 'gyro_z'
        INTERPOLATED_COLUMN = 'interpolated'
        INTERPOLATED_GYRO_COLUMN = 'interpolated_gyro'
        IS_PRACTICE_COLUMN = 'is_practice'
        NUMERICAL_QUESTION_TYPES = ['likert']
        CATEGORICAL_QUESTION_TYPES = ['multiple','text_input']
        PARTICIPANT_COLUMN = 'participant'
        PEAK_AT_COLUMN = 'peak_at'
        RESPONSE_COLUMN = 'response'
        RT_COLUMN = 'rt'
        RT_INV_COLUMN = 'rt_inv'
        SENSOR_TYPE_COLUMN = 'sensor_type'
        SESSION_COLUMN = "session"
        SESSION_NUMBER_COLUMN = "session_number"
        SIGNED_UP_COLUMN = 'signed_up'
        STIMULUS_COLUMN = 'stimulus'
        STIMULUS_SET_COLUMN = 'stimulus_set'
        TIME_COLUMN = 'times'
        TIME_CORRECTION_COLUMN = 'time_correction'
        TRIAL_NUMBER_COLUMN = 'block_trial'
        TRIAL_NUMBER_CUM_COLUMN = 'trial'
        V_FINAL_COLUMN = 'v_final'
        V_MAX_COLUMN = 'v_max'


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/01_json_to_df.ipynb (unless otherwise specified).

__all__ = ['get_sensor_data', 'addAATData', 'addSessionData', 'makeParticipantTable', 'relabel_variables', 'json_to_df']

# Cell
from .utils import loadJson
from .constants import Constants as C
from .add_responses import add_responses
import json
import os
import pandas as pd
from pandas.api.types import is_numeric_dtype
import warnings
import numpy as np
from tqdm import tqdm
from operator import itemgetter
warnings.filterwarnings("ignore", message="numpy.dtype size changed")
warnings.filterwarnings("ignore", message="numpy.ufunc size changed")

# Cell
def get_sensor_data(data, key):
    if key in data:
        sensor_data = pd.Series(data[key])
        sensor_data.index = sensor_data.index.astype('int')
        sensor_data = sensor_data.sort_index()
        times = sensor_data.index.values
        sensor_data = sensor_data.values
    else:
        times, sensor_data = np.array([]),np.array([])
    return sensor_data, times

# Cell
def addAATData(df, session, sessionName, task, inverted_stimulus_sets):
    for blockIndex, block in enumerate(session[task]['blocks'], 1):
        for trialIndex, trial in enumerate(block[1:], 1):
            if trial == None:
                continue
            # Note that extra practice trials get skipped
            if (sessionName,blockIndex,trialIndex) in df.index:
                # Setting trial variables
                row = (sessionName,blockIndex,trialIndex)
                df.at[row,C.CORRECT_RESPONSE_COLUMN] = trial['correctResponse'] # 1 = PUSH, 2 = PULL
                image_name = trial['imageName'].split('.')
                image_name = image_name[0] if isinstance(image_name, list) else np.nan
                df.at[row,C.STIMULUS_COLUMN] = image_name#trial['imageName'].split('.')#.apply(lambda x: x[0] if isinstance(x, list) else np.nan)
                df.at[row,C.DRAWN_AT_UNIX_COLUMN] = trial.get('drawnAtUnix', np.nan)
                df.at[row,C.DRAWN_AT_COLUMN] = trial.get('drawnAt', np.nan)
                df.at[row,C.STIMULUS_SET_COLUMN] = inverted_stimulus_sets.get(trial.get('imageName',None), np.nan)
                acc, times = get_sensor_data(trial, 'acceleration')
                df.at[row,C.TIME_COLUMN] = times
                df.at[row,C.ACCELERATION_COLUMN] = acc
                df.at[row,C.ACCELERATION_X_COLUMN] = get_sensor_data(trial, 'acceleration_x')[0]
                df.at[row,C.ACCELERATION_Y_COLUMN] = get_sensor_data(trial, 'acceleration_y')[0]
                gyro_x, gyro_times = get_sensor_data(trial, 'gyro_x')
                df.at[row,C.GYRO_TIME_COLUMN] = gyro_times
                df.at[row,C.GYRO_X_COLUMN] = gyro_x
                df.at[row,C.GYRO_Y_COLUMN] = get_sensor_data(trial, 'gyro_y')[0]
                df.at[row,C.GYRO_Z_COLUMN] = get_sensor_data(trial, 'gyro_z')[0]
    return df

# Cell
def addSessionData(df, ppJson, sessionName, inverted_stimulus_sets):
    for task in sorted(ppJson[sessionName].keys()):
        if task == "AAT":
            df = addAATData(df, ppJson[sessionName], sessionName, task, inverted_stimulus_sets)
        elif task.startswith('PR'):
            for stimulus_name in ppJson[sessionName][task].keys():
                df.loc[df.stimulus==stimulus_name,task] = ppJson[sessionName][task][stimulus_name]['rating']
                #ratingDf.loc[stimulus_name,task] = ppJson[sessionName][task][stimulus_name]['rating']
        else:
            for question in ppJson[sessionName][task].keys():
                answer = ppJson[sessionName][task][question]['answer']
                if 'time' in ppJson[sessionName][task][question]:
                    time = ppJson[sessionName][task][question]['time']
                else:
                    time = None
                if is_numeric_dtype(df[question]):
                    answer = float(answer)
                df.loc[sessionName, question] = answer
                df.loc[sessionName, question + "_time"] = time
    return df

# Cell
def makeParticipantTable(participantJson, df, inverted_stimulus_sets):
    '''This function creates dataframes for participants'''

    # Sometimes participantIds miss from the data
    participantKeys = { 'participantId':'participant',
                            'experiment':'experiment',
                            'device':'device',
                            'signed_up':'signed_up',
                            'sensor_type':'sensor_type',
                            'condition':'condition',
                            'saved_at':'saved_at',
                            'completion':'completion'}
    # Adding participant level data
    for jsonKey, tableColumn in participantKeys.items():
        if jsonKey in participantJson.keys():
            df[tableColumn] = participantJson[jsonKey]
    # Adding session level DataFrame
    keys = participantJson.keys()
    has_pr_list = []
    for key in participantJson.keys():
        if key not in participantKeys:
            has_pr = 0
            for task in sorted(participantJson[key].keys()):
                if task.startswith("PR"):
                    has_pr = 1
            has_pr_list.append([key,has_pr])
    has_pr_list = sorted(has_pr_list, key=itemgetter(1))
    keys = [item[0] for item in has_pr_list]

    for key in keys:#participantJson.keys():
        if key not in participantKeys:
            df = addSessionData(df, participantJson,key, inverted_stimulus_sets)
    df[C.CORRECT_RESPONSE_COLUMN].replace({1:'push',2:'pull'}, inplace=True)
    #ratingDf[C.PARTICIPANT_COLUMN] = participantJson['participantId']
    return df

# Cell
def relabel_variables(data, tasks):
    labels = {}
    for task_item in tasks.items():
        task_name = task_item[0]
        task = task_item[1]
        task_type = task.get("type","")
        if task_type=="questionnaire":
            participant_questionnaire = task.get("level", "") == "participant"

            questions = task.get("questions",[])
            for index, question in enumerate(questions):
                if "id" not in question:
                    question["id"] = "%s_%02d"%(task_name,index+1)
                if (question.get("level","") == "participant") or participant_questionnaire:
                    if question['id'] in data:
                        data[question['id']] = data[question['id']].ffill().bfill() # Distributing participant variables

                if "label" in question:
                    #if "id" in question:
                    labels[question['id']] = question['label'] # Relabelling variables
                    #else:
                    #    labels["%s_%02d"%(task_name,index+1)]

    for item in labels.items():
        data[item[1]] = data[item[0]]
    return data

# Cell
def json_to_df(input_dir,external_dir,interim_dir,condition_dict, limit= None, threshold = 1):
    stimulus_sets = loadJson(os.path.join(external_dir, "stimulus_sets.json"))
    tasks = loadJson(os.path.join(external_dir, "tasks.json"))
    inverted_stimulus_sets = dict( (v,k) for k in stimulus_sets for v in stimulus_sets[k] )
    errors = []
    for file in tqdm(list(os.scandir(input_dir))[:limit]):
            participant = file.name.split('.')[0]
            if file.name.endswith('.json'):
                participantJson = loadJson(file.path)
                if ('condition' in participantJson.keys()) & ('participantId' in participantJson.keys()):
                    conditionDf = condition_dict[participantJson['condition']]
                    conditionDf = conditionDf.copy(deep=True)
                    df = makeParticipantTable(participantJson, conditionDf, inverted_stimulus_sets)
                    df = df.reset_index().set_index(['participant','session','block','trial'])
                    df = add_responses(df, threshold = threshold)
                    #df = relabel_variables(df, tasks)

                    df.reset_index().to_pickle(os.path.join(interim_dir, "%s.pkl"%participant))


    #assert len(dfs)>0, "There is no data in input_dir of json_to_df. You might have to specify a different input dir in make_dataset.py."
    #complete_data = pd.concat(dfs, sort=True).reset_index()
    #complete_data['stimulus'] = complete_data.stimulus.str.split('.').apply(lambda x: x[0] if isinstance(x, list) else np.nan)
    #complete_rating = pd.concat(ratingDfs, sort=True)
    #complete_rating = complete_rating.reset_index().rename(columns={'index':'stimulus'})#.set_index(['participant','rating'])
    #complete_data = complete_data.merge(complete_rating, on = ['participant','stimulus'], how = 'left')
    #if len(errors):
    #    print(errors)
    #return complete_data


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/04_utils.ipynb (unless otherwise specified).

__all__ = ['merge_data', 'loadJson']

# Cell
import json # 2.0.9
import numpy as np
import glob
import pandas as pd
import os

def merge_data(folder, drop=[], limit = None):
    all_files = glob.glob(folder + "*.pkl")
    li = []
    for filename in all_files[:limit]:
        df = pd.read_pickle(filename)
        df = df.drop(columns = drop)
        li.append(df)
    data = pd.concat(li, axis=0, ignore_index=True)
    return data

def loadJson(path, default = {}):
    """Return dictionary from json file."""
    if os.path.isfile(path):
        try:
            with open(path, 'r', encoding = 'utf-8') as f:
                return json.loads(f.read(), strict = False)
        except:
            return default
    else:
        return default


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/05_make_condition_templates.ipynb (unless otherwise specified).

__all__ = ['sorted_nicely', 'getProperty', 'makeConditionTable', 'make_condition_templates_from_files',
           'make_condition_templates', 'BASE_COLUMNS', 'AAT_COLUMNS']

# Cell
from .utils import loadJson
from .constants import Constants as C
import json
import os
import numpy as np
import pandas as pd
import re

BASE_COLUMNS = [C.PARTICIPANT_COLUMN, C.DEVICE_COLUMN, C.EXPERIMENT_COLUMN, C.CONDITION_COLUMN, C.SIGNED_UP_COLUMN, C.SESSION_COLUMN]
AAT_COLUMNS = [C.SENSOR_TYPE_COLUMN, C.BLOCK_COLUMN, C.TRIAL_NUMBER_COLUMN, C.TRIAL_NUMBER_CUM_COLUMN, C.IS_PRACTICE_COLUMN, C.STIMULUS_SET_COLUMN, C.STIMULUS_COLUMN, C.CORRECT_RESPONSE_COLUMN, C.TIME_COLUMN, C.ACCELERATION_COLUMN,C.ACCELERATION_X_COLUMN,C.ACCELERATION_Y_COLUMN, C.GYRO_TIME_COLUMN, C.GYRO_X_COLUMN,C.GYRO_Y_COLUMN,C.GYRO_Z_COLUMN,C.INTERPOLATED_COLUMN, C.INTERPOLATED_GYRO_COLUMN, C.DRAWN_AT_UNIX_COLUMN, C.DRAWN_AT_COLUMN]

def sorted_nicely(l):
    """Return list of sorted strings containing numbers (e.g. PI8, PI9, PI10, ...)"""
    convert = lambda text: int(text) if text.isdigit() else text
    alphanum_key = lambda key: [convert(c) for c in re.split('([0-9]+)', key)]
    return sorted(l, key = alphanum_key)

# Creates columns and lines based on condition
def getProperty(object, property, object_dict, default = np.nan):
    object = object_dict[object]
    if property in object.keys() and object[property]!=None:
        return object[property]
    elif "parent" in object.keys():
        return getProperty(object['parent'], property, object_dict, default=default)
    # Added functionality for "parents"
    elif "parents" in object.keys():
        for parent in object['parents']:
            parent_property = getProperty(parent, property, object_dict, default=default)
            if parent_property != default:
                return parent_property
    else:
        return default

def makeConditionTable(condition_name,conditions_file, sessions_file, tasks_file, stimulus_sets_file, blocks_file):
    columns = BASE_COLUMNS
    question_columns, numeric_question_columns, categorical_question_columns, block_numbers, trial_numbers, trial_numbers_cum, session_list, session_number_list, is_practice_list = [], [], [], [], [], [], [], [], []
    conditions = loadJson(conditions_file)
    condition = conditions[condition_name]
    sessions = loadJson(sessions_file)
    tasks = loadJson(tasks_file)
    stimulus_sets = loadJson(stimulus_sets_file)
    blocks = loadJson(blocks_file, default = None)
    for session_number, session in enumerate(condition['sessions']):
        has_aat = False
        task_names = getProperty(session, "tasks", sessions)
        for task in task_names:
            type = getProperty(task, "type", tasks)
            # For questionnaires and picture ratings add colums to the data table
            if type == 'questionnaire':
                questions = getProperty(task, "questions", tasks)
                for index, question in enumerate(questions):
                    if isinstance(question, str):
                        question = {"text":question}
                    if "type" in question.keys():
                        question_format = question['type']['format']
                    else:
                        question_format = getProperty(task, 'default_type', tasks)['format']
                    if question_format != "instruction":
                        if "id" in question.keys():
                            question_id = question['id']
                        else:
                            question_id = "%s_%02d"%(task,index+1)
                        question_columns.append(question_id)
                        question_columns.append(question_id+"_time")
                        categorical_question_columns.append(question_id+"_time")
                        if question_format in C.NUMERICAL_QUESTION_TYPES:
                            numeric_question_columns.append(question_id)
                        if question_format in C.CATEGORICAL_QUESTION_TYPES:
                            categorical_question_columns.append(question_id)
            # For AATs check how many trials there are in each block (based on stimulus sets and repetitions; TODO: This should also include repeat stimuli option)
            elif type == "aat":
                has_aat = True
                aat = tasks[task]
                if "blocks" in aat:
                    cum_trial = 1
                    for block_number, block_name in enumerate(aat["blocks"]):
                        block = blocks[block_name]
                        is_practice = False
                        if "give_feedback" in block:
                            is_practice = block["give_feedback"]
                        amount_of_trials = 0
                        for response in ["push","pull"]:
                            response_definition = block[response]
                            for chooser in response_definition["stimuli"]:
                                stim_set = chooser["from"]
                                repeat = 1
                                if "repeat" in chooser:
                                    repeat = chooser["repeat"]
                                amount_of_trials += chooser["pick"] * repeat
                        for trial_number in range(0, int(amount_of_trials)):
                            session_list.append(session)
                            session_number_list.append(session_number+1)
                            block_numbers.append(block_number+1)
                            trial_numbers.append(trial_number+1)
                            trial_numbers_cum.append(cum_trial)
                            is_practice_list.append(is_practice)
                            cum_trial += 1
                                #print(stim_set)

                else:
                    # Counting practice trials
                    practice_size = 0
                    for stim_set in getProperty(task, "practice_targets", tasks, default = []):
                        practice_size += len(stimulus_sets[stim_set])
                    for stim_set in getProperty(task, "practice_controls", tasks, default = []):
                        practice_size += len(stimulus_sets[stim_set])
                    stim_size = 0
                    # Counting experimental trials
                    for stim_set in getProperty(task, "targets", tasks, default = []):
                        stim_size += len(stimulus_sets[stim_set]) * getProperty(task, 'target_rep', tasks, default= 1)
                    for stim_set in getProperty(task, "controls", tasks, default = []):
                        stim_size += len(stimulus_sets[stim_set]) * getProperty(task, 'control_rep', tasks, default= 1)
                    amount_of_blocks = getProperty(task, "amount_of_blocks", tasks)
                    cum_trial = 1
                    for block_number in range(amount_of_blocks):
                        # If it's a practice block
                        if (block_number == 0 or block_number == amount_of_blocks/2):
                            amount_of_trials = practice_size/2
                            is_practice = True
                        else:
                            amount_of_trials = stim_size/(amount_of_blocks-2)
                            is_practice = False
                        for trial_number in range(0, int(amount_of_trials)):
                            session_list.append(session)
                            session_number_list.append(session_number+1)
                            block_numbers.append(block_number+1)
                            trial_numbers.append(trial_number+1)
                            trial_numbers_cum.append(cum_trial)
                            is_practice_list.append(is_practice)
                            cum_trial += 1
        if not has_aat:
            session_list.append(session)
            session_number_list.append(session_number+1)
            block_numbers.append(1)
            trial_numbers.append(1)
            trial_numbers_cum.append(1)
            is_practice_list.append(False)
    columns = columns + sorted_nicely(set(question_columns)) + AAT_COLUMNS
    numeric_question_columns = list(set(numeric_question_columns))
    categorical_question_columns = list(set(categorical_question_columns))

    df = pd.DataFrame(columns=columns, index = list(range(len(trial_numbers))))

    df[C.ACCELERATION_COLUMN] = df[C.ACCELERATION_COLUMN].apply(lambda x: [])
    df[C.ACCELERATION_X_COLUMN] = df[C.ACCELERATION_X_COLUMN].apply(lambda x: [])
    df[C.ACCELERATION_Y_COLUMN] = df[C.ACCELERATION_Y_COLUMN].apply(lambda x: [])
    df[C.GYRO_X_COLUMN] = df[C.GYRO_X_COLUMN].apply(lambda x: [])
    df[C.GYRO_Y_COLUMN] = df[C.GYRO_Y_COLUMN].apply(lambda x: [])
    df[C.GYRO_Z_COLUMN] = df[C.GYRO_Z_COLUMN].apply(lambda x: [])
    df[C.TIME_COLUMN] = df[C.TIME_COLUMN].apply(lambda x: [])
    df[C.GYRO_TIME_COLUMN] = df[C.GYRO_TIME_COLUMN].apply(lambda x: [])
    df[C.CONDITION_COLUMN] = condition_name
    df[C.SESSION_COLUMN] = session_list
    df[C.SESSION_NUMBER_COLUMN] = session_number_list
    df[C.BLOCK_COLUMN] = block_numbers
    df[C.IS_PRACTICE_COLUMN] = is_practice_list
    df[C.IS_PRACTICE_COLUMN] = df[C.IS_PRACTICE_COLUMN].astype(bool)
    df[C.TRIAL_NUMBER_COLUMN] = pd.Series(trial_numbers)
    df[C.TRIAL_NUMBER_CUM_COLUMN] = pd.Series(trial_numbers_cum)
    df[C.INTERPOLATED_COLUMN] = df[C.INTERPOLATED_COLUMN].astype('str')
    df[C.INTERPOLATED_GYRO_COLUMN] = df[C.INTERPOLATED_GYRO_COLUMN].astype('str')
    df[numeric_question_columns] = df[numeric_question_columns].astype(float)
    # Changing nan to None in String colums to facilitate conversion to pandas
    df[categorical_question_columns] = df[categorical_question_columns].where(pd.notnull(df[categorical_question_columns]),None)
    df = df.set_index([C.SESSION_COLUMN, C.BLOCK_COLUMN, C.TRIAL_NUMBER_COLUMN]).sort_index()
    return df


def make_condition_templates_from_files(conditions_file, sessions_file, tasks_file, stimulus_sets_file, blocks_file):

    # Getting condition
    condition_dict = {}
    conditions = loadJson(conditions_file)
    for condition in conditions.keys():
        condition_table = makeConditionTable(condition,conditions_file, sessions_file, tasks_file, stimulus_sets_file, blocks_file)
        condition_dict[condition] = condition_table
    return condition_dict

def make_condition_templates(external_folder):
    return make_condition_templates_from_files(os.path.join(external_folder, "conditions.json"),
                                      os.path.join(external_folder, "sessions.json"),
                                      os.path.join(external_folder, "tasks.json"),
                                      os.path.join(external_folder, "stimulus_sets.json"),
                                      os.path.join(external_folder, "blocks.json"));


# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/03_add_responses.ipynb (unless otherwise specified).

__all__ = ['interpolate', 'butterFilter', 'find_first_peak', 'find_response', 'getResponses', 'add_responses']

# Cell
from .constants import Constants as C
import numpy as np # 1.18.4
import pandas as pd #'1.0.3'
import peakutils #'1.3.3'
from scipy.interpolate import splrep, splev # 1.4.1
from scipy.signal import butter, filtfilt
from tqdm import tqdm #'4.46.0'

def interpolate(times, accelerations):
    interpolated = np.array([])
    # Times sometimes comes in as a nan value requring this try except
    try:
        if len(times):
            spl = splrep(times, accelerations)
            x2 = np.arange(times[0], times[-1])
            interpolated = splev(x2, spl)
    except:
        pass#
    return interpolated


def butterFilter(acc, cutoff, frequency, order):
    nyq = 0.5 * frequency
    normal_cutoff = cutoff / nyq
    b, a = butter(order, normal_cutoff, btype='low', analog=False)
    buttered = filtfilt(b, a, acc)
    return buttered


def find_first_peak(acceleration, threshold, min_dist):
    first_peak, force, response = np.nan, np.nan, "NA" # If no acceleration present, set response to NA
    if (len(acceleration)>0):
        peaks = peakutils.peak.indexes(abs(acceleration), thres=.3, min_dist=min_dist,thres_abs=False)
        if len(peaks): # If peaks were detected
            first_peak = peaks[0]
            force = acceleration[first_peak]
            if force > threshold: # Set response to pull if bigger than threshold
                response = "pull"
            elif force < -threshold: # Set response to push if smaller than threshold
                response = "push"
            else: # If response did not pass threshold, set it to timeout
                first_peak = np.nan
                force = np.nan
                response = "none"
        else:
            response = "ND" # If no peaks are detected set response to not determined
    return (first_peak, force, response)


def find_response(response, acceleration, first_peak):
    rt = np.nan
    if not np.isnan(first_peak):
        rts = np.where((abs(acceleration[1:first_peak-10])<.5))[0]
        if len(rts):
            rt = rts[-1]
        else:
            response = "ND"
    return rt, response


def getResponses(line, min_dist, threshold, butter_filter):
    # Smooth the data to millisecond resolution
    acceleration = interpolate(line.loc[C.TIME_COLUMN], line.loc[C.ACCELERATION_COLUMN])
    # Butter filter
    if butter_filter == True:
        acceleration = butterFilter(acceleration, 10, 1000, 2)
    # Find the first peak (location and magnitude)
    first_peak, force, response = find_first_peak(acceleration, threshold, min_dist)
    # Finding the reaction time
    rt, response = find_response(response, acceleration, first_peak)
    # Reduce size of acceleration column
    acceleration = acceleration.astype('float16')
    output = [response, rt, abs(force), first_peak]
    output.append(acceleration)
    gyro = interpolate(line.loc[C.GYRO_TIME_COLUMN], line.loc[C.GYRO_Y_COLUMN])
    gyro = gyro.astype('float16')
    output.append(gyro)
    return output


def add_responses(df, min_dist = 10, threshold = 1, butter_filter = False):
    # Initiating progress bar
    #tqdm.pandas()
    statistics = [C.RESPONSE_COLUMN,
                  C.RT_COLUMN,
                  C.FORCE_COLUMN,
                  C.PEAK_AT_COLUMN,
                  C.INTERPOLATED_COLUMN,
                  C.INTERPOLATED_GYRO_COLUMN]

    df[statistics] = df.apply(getResponses,axis=1,result_type="expand", min_dist = min_dist, threshold=threshold, butter_filter=butter_filter)
    df[C.ACCURACY_COLUMN] = df.response == df.correct_response
    df[C.RT_INV_COLUMN] = 1000/df[C.RT_COLUMN]
    df = df.drop(columns=['level_0','times','accelerations','accelerations_x',
       'accelerations_y','gyro_times','gyro_x', 'gyro_y', 'gyro_z'], errors='ignore')
    # Changin null values to make operations in R easier
    categorical_question_columns = ['condition','correct_response', 'device','drawn_at','experiment','sensor_type','stimulus','stimulus_set']
    df[categorical_question_columns] = df[categorical_question_columns].where(pd.notnull(df[categorical_question_columns]),None)
    numeric_question_columns = ['drawn_at', 'drawn_at_unix']
    df[numeric_question_columns] = df[numeric_question_columns].astype(float)
    df['is_pull'] = df.response == 'pull'
    return df.reset_index(drop = False)